;==========================================================================
; resh2inc.exe reads a resource.h file and creates a resource.inc file.
; This permits you to use a header file in your resource definition and
; also use those same #defines as equates in your MASM32 program.
; simply modify the MASM32 bat files to use this program.  This program 
; will comment out non definitions like #ifdef etc., it also removes all
; C/C++ block comments and Multi line comments. 
; Send suggestions or comments to Dan
; danesmith@comcast.net
;==========================================================================
      .386
      .model flat, stdcall  ; 32 bit memory model
      option casemap :none  ; case sensitive
      IS_DEBUG EQU 1
;--------------------------------------------------------------------------
;     include files
;--------------------------------------------------------------------------
      include \MASM32\INCLUDE\windows.inc
      include \MASM32\INCLUDE\masm32.inc
      include \MASM32\INCLUDE\gdi32.inc
      include \MASM32\INCLUDE\user32.inc
      include \MASM32\INCLUDE\kernel32.inc
      include \MASM32\INCLUDE\Comctl32.inc
      include \MASM32\INCLUDE\comdlg32.inc
      include \MASM32\INCLUDE\oleaut32.inc
;--------------------------------------------------------------------------
;     libraries
;--------------------------------------------------------------------------
      includelib \MASM32\LIB\masm32.lib
      includelib \MASM32\LIB\gdi32.lib
      includelib \MASM32\LIB\user32.lib
      includelib \MASM32\LIB\kernel32.lib
      includelib \MASM32\LIB\Comctl32.lib
      includelib \MASM32\LIB\comdlg32.lib
      includelib \MASM32\LIB\oleaut32.lib
;--------------------------------------------------------------------------
      ReadHeader    PROTO 
      FormatInclude PROTO
      WriteInclude  PROTO

.data
    len         dd 0
    InMem       dd 0
    OutMem      dd 0
    Equate      db 'equ '
    inResource  db 'resource.h',0
    outResource db 'resource.inc',0

.code

;======================================================================

start:
    invoke ReadHeader
    .if eax > 0
        invoke FormatInclude
        invoke WriteInclude
    .endif
    mov eax,0
    invoke ExitProcess, eax

;==========================================================================

ReadHeader proc
    LOCAL hFile :DWORD
    LOCAL br    :DWORD

    invoke CreateFile,addr inResource,GENERIC_READ,FILE_SHARE_READ,
            NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL
    mov hFile, eax                      ; Open input file and get handle
    .if eax == 0ffffffffh
        mov eax,0
        ret
    .endif
    

    invoke GetFileSize,hFile,NULL       ; get its size in bytes
    mov len, eax

    invoke SysAllocStringByteLen,0,len  ; get memory for it
    mov InMem, eax

    invoke ReadFile,hFile,InMem,len,ADDR br,NULL
    invoke CloseHandle,hFile            ; read the file and clean up

    mov eax,1
    ret
ReadHeader endp

;==========================================================================
FormatInclude proc
    LOCAL state:DWORD

    mov eax,len                ; get one more
    inc eax                    ; byte for null char
    invoke SysAllocStringByteLen,0,eax
    mov OutMem, eax            ; get some memory in which to put the output

    MOVECHAR  equ 1            ; pick a few STATE values for this engine
    SKIPDEF   equ 2
    MOVESPACE equ 3
    MOVENAME  equ 4

    mov state,MOVECHAR         ; start out in the move char state
    mov eax,InMem              ; set up our pointers
    mov ebx,OutMem
    mov edx,0                  ; clear work register
    mov ecx,0                  ; clear count register
    .while ecx < len                       ; and begin to loop
        mov dl,[eax]                       ; get the first character
        mov dh,0                           ; clear look ahead character
        inc ecx                            ; count the character
        inc eax                            ; and skip it
        .if state == MOVECHAR           ; is this the move chracter state?
            .if dl == '/'                 ; is this a possible comment?
                mov dh,[eax]               ; then peek at next char
                .if dh == '/'              ; is this a block C++ comment
                    mov dl,';'             ; then make it an ASM comment
                    mov [ebx],dl           ; write the new char
                    inc ebx                ; and skip it
                    .continue              ; before going on
                .endif
                .if dh == '*'             ; is this a multi line C comment?  
                    inc eax                ; skip the * character
                    inc ecx                ; and count it
                    .while ecx < len       ; now search to find the end */
                        mov dl,[eax]         ; get the char
                        inc eax              ; and skip it
                        inc ecx              ; count it too
                        .if dl == '*'        ; is it a possible end?
                           mov dh,[eax]        ; then peek at next char
                           .if dh == '/'       ; is it the end?    
                              inc eax            ; then skip
                              inc ecx            ; and count it
                              .break             ; and continue outer loop
                           .endif
                        .endif
                    .endw
                    .continue              ; go get next char 
                .endif
            .endif
            .if dl == '#'                  ; is this a possible define?
                mov dh,[eax]                 ; then peek at next char
                .if dh == 'd' || dh == 'D'   ; could check for "define" here
                    mov state,SKIPDEF      ; if define then skip it
                    inc eax                  ; skip this
                    inc ecx                  ; and count it
                    .continue                ; then look for next char
                .else
                    mov dl,';'             ; non defines become comments
                    mov [ebx],dl           ; overlay # with a ;
                .endif
            .endif
            mov [ebx],dl                   ; move all other chars 
            inc ebx                        ; prepare for next char                                     
            .continue                      ; and get next char
        .endif
        .if state == SKIPDEF               ; are we skipping the definition?
            .if dl == ' ' || dl == 9 || dl == 13
                mov state,MOVESPACE        ; if so and we have whitespace
                mov [ebx],dl               ; change state, move the white space
                inc ebx                    ; prepare for next char 
                .continue                  ; and go get it
            .endif
        .endif
        .if state == MOVESPACE             ; we are moving spaces between
            mov [ebx],dl                   ; #defines and their names
            inc ebx                        ; so move the char and prepare for next
            .if dl != ' ' && dl != 9 && dl != 13
               mov state,MOVENAME          ; we found the name of the equate
            .endif                         ; mark the name state and 
            .continue                      ; go move it's chars
        .endif
        .if state == MOVENAME              ; here we are moving the equate name
            mov [ebx],dl                   
            inc ebx
            .if dl == ' ' || dl == 9 || dl == 13
                push ebp
                lea ebp,Equate             ; when white space is found move 'equ '
                mov ebp,[ebp]
                mov [ebx],ebp
                add ebx,3                  ; don't count space from equ
                pop ebp
                mov [ebx],dl               ; but move the whitespace we found
                inc ebx
                mov state,MOVECHAR         ; now, go back to moving all chars
            .endif
        .endif
    .endw
    mov dl,0                              ; null terminate the output
    mov [ebx],dl                           
    invoke lnstr,OutMem                   ; get the new length
    mov len,eax
    invoke SysFreeString,InMem            ; free input memory and return
    ret
FormatInclude endp

;==========================================================================

WriteInclude proc
    LOCAL hFile :DWORD
    LOCAL bw    :DWORD

    invoke CreateFile,addr outResource, ; pointer to name of the file
            GENERIC_WRITE,              ; access (read-write) mode
            NULL,                       ; share mode
            NULL,                       ; pointer to security attributes
            CREATE_ALWAYS,              ; how to create
            FILE_ATTRIBUTE_NORMAL,      ; file attributes
            NULL
    mov hFile,eax
    

    invoke WriteFile,hFile,OutMem,len,ADDR bw,NULL

    invoke SysFreeString,OutMem          ; free memory after the write
    invoke CloseHandle,hFile             ; clean up the file

    ret                                  ; and return

WriteInclude endp

end start
